// SPDX-License-Identifier: GPL-3.0-or-later
// Syscall dispatcher
// This file was generated by scripts/gen_syscall.py

#pragma once

#include <mos/filesystem/fs_types.h>
#include <mos/io/io_types.h>
#include <mos/mos_global.h>
#include <mos/tasks/signal_types.h>
#include <mos/types.h>
#include <sys/poll.h>
#include <sys/select.h>
#include <sys/uio.h>

// syscall implementation declarations and syscall numbers
#include <mos/syscall/decl.h>
#include <mos/syscall/number.h>

// debugging support
#include "mos/syslog/printk.h"

should_inline reg_t dispatch_syscall(const reg_t number, reg_t arg1, reg_t arg2, reg_t arg3, reg_t arg4, reg_t arg5, reg_t arg6)
{
    MOS_UNUSED(arg1);
    MOS_UNUSED(arg2);
    MOS_UNUSED(arg3);
    MOS_UNUSED(arg4);
    MOS_UNUSED(arg5);
    MOS_UNUSED(arg6);
    
    reg_t ret = -ENOSYS;
    
    switch (number)
    {
        case SYSCALL_vfs_openat:
        {
            pr_dinfo2(syscall, "vfs_openat(dirfd=%d, file_path='%s', flags=%x)", (fd_t) arg1, (const char *) arg2, (open_flags) arg3);
            ret = (reg_t) impl_syscall_vfs_openat((fd_t) arg1, (const char *) arg2, (open_flags) arg3);
            break;
        }
        case SYSCALL_vfs_fstatat:
        {
            pr_dinfo2(syscall, "vfs_fstatat(dirfd=%d, file_path='%s', stat_buf=%p, flags=%x)", (fd_t) arg1, (const char *) arg2, (void *) arg3, (fstatat_flags) arg4);
            ret = (reg_t) impl_syscall_vfs_fstatat((fd_t) arg1, (const char *) arg2, (file_stat_t *) arg3, (fstatat_flags) arg4);
            break;
        }
        case SYSCALL_io_read:
        {
            pr_dinfo2(syscall, "io_read(fd=%d, buffer=%p, size=%zu)", (fd_t) arg1, (void *) arg2, (size_t) arg3);
            ret = (reg_t) impl_syscall_io_read((fd_t) arg1, (void *) arg2, (size_t) arg3);
            break;
        }
        case SYSCALL_io_write:
        {
            pr_dinfo2(syscall, "io_write(fd=%d, buffer=%p, size=%zu)", (fd_t) arg1, (void *) arg2, (size_t) arg3);
            ret = (reg_t) impl_syscall_io_write((fd_t) arg1, (const void *) arg2, (size_t) arg3);
            break;
        }
        case SYSCALL_io_close:
        {
            pr_dinfo2(syscall, "io_close(fd=%d)", (fd_t) arg1);
            ret = (reg_t) impl_syscall_io_close((fd_t) arg1);
            break;
        }
        case SYSCALL_exit:
        {
            pr_dinfo2(syscall, "exit(exit_code=%u)", (u32) arg1);
            impl_syscall_exit((u32) arg1);
            break;
        }
        case SYSCALL_yield_cpu:
        {
            pr_dinfo2(syscall, "yield_cpu()");
            impl_syscall_yield_cpu();
            break;
        }
        case SYSCALL_fork:
        {
            pr_dinfo2(syscall, "fork()");
            ret = (reg_t) impl_syscall_fork();
            break;
        }
        case SYSCALL_get_pid:
        {
            pr_dinfo2(syscall, "get_pid()");
            ret = (reg_t) impl_syscall_get_pid();
            break;
        }
        case SYSCALL_get_parent_pid:
        {
            pr_dinfo2(syscall, "get_parent_pid()");
            ret = (reg_t) impl_syscall_get_parent_pid();
            break;
        }
        case SYSCALL_spawn:
        {
            pr_dinfo2(syscall, "spawn(file_path='%s', argv=%p, envp=%p)", (const char *) arg1, (void *) arg2, (void *) arg3);
            ret = (reg_t) impl_syscall_spawn((const char *) arg1, (const char *const *) arg2, (const char *const *) arg3);
            break;
        }
        case SYSCALL_create_thread:
        {
            pr_dinfo2(syscall, "create_thread(name='%s', entry=%p, arg=%p, stack_size=%zu, stack=%p)", (const char *) arg1, (void *) arg2, (void *) arg3, (size_t) arg4, (void *) arg5);
            ret = (reg_t) impl_syscall_create_thread((const char *) arg1, (thread_entry_t) arg2, (void *) arg3, (size_t) arg4, (void *) arg5);
            break;
        }
        case SYSCALL_get_tid:
        {
            pr_dinfo2(syscall, "get_tid()");
            ret = (reg_t) impl_syscall_get_tid();
            break;
        }
        case SYSCALL_thread_exit:
        {
            pr_dinfo2(syscall, "thread_exit()");
            impl_syscall_thread_exit();
            break;
        }
        case SYSCALL_wait_for_thread:
        {
            pr_dinfo2(syscall, "wait_for_thread(tid=%d)", (tid_t) arg1);
            ret = (reg_t) impl_syscall_wait_for_thread((tid_t) arg1);
            break;
        }
        case SYSCALL_futex_wait:
        {
            pr_dinfo2(syscall, "futex_wait(futex=%p, val=%u)", (void *) arg1, (u32) arg2);
            ret = (reg_t) impl_syscall_futex_wait((futex_word_t *) arg1, (u32) arg2);
            break;
        }
        case SYSCALL_futex_wake:
        {
            pr_dinfo2(syscall, "futex_wake(futex=%p, count=%zu)", (void *) arg1, (size_t) arg2);
            ret = (reg_t) impl_syscall_futex_wake((futex_word_t *) arg1, (size_t) arg2);
            break;
        }
        case SYSCALL_ipc_create:
        {
            pr_dinfo2(syscall, "ipc_create(name='%s', max_pending_connections=%zu)", (const char *) arg1, (size_t) arg2);
            ret = (reg_t) impl_syscall_ipc_create((const char *) arg1, (size_t) arg2);
            break;
        }
        case SYSCALL_ipc_accept:
        {
            pr_dinfo2(syscall, "ipc_accept(fd=%d)", (fd_t) arg1);
            ret = (reg_t) impl_syscall_ipc_accept((fd_t) arg1);
            break;
        }
        case SYSCALL_ipc_connect:
        {
            pr_dinfo2(syscall, "ipc_connect(name='%s', buffer_size=%zu)", (const char *) arg1, (size_t) arg2);
            ret = (reg_t) impl_syscall_ipc_connect((const char *) arg1, (size_t) arg2);
            break;
        }
        case SYSCALL_arch_syscall:
        {
            pr_dinfo2(syscall, "arch_syscall(nr=%llu, arg1=%llu, arg2=%llu, arg3=%llu, arg4=%llu)", (u64) arg1, (u64) arg2, (u64) arg3, (u64) arg4, (u64) arg5);
            ret = (reg_t) impl_syscall_arch_syscall((u64) arg1, (u64) arg2, (u64) arg3, (u64) arg4, (u64) arg5);
            break;
        }
        case SYSCALL_vfs_mount:
        {
            pr_dinfo2(syscall, "vfs_mount(device='%s', mount_point='%s', fs_type='%s', options='%s')", (const char *) arg1, (const char *) arg2, (const char *) arg3, (const char *) arg4);
            ret = (reg_t) impl_syscall_vfs_mount((const char *) arg1, (const char *) arg2, (const char *) arg3, (const char *) arg4);
            break;
        }
        case SYSCALL_vfs_readlinkat:
        {
            pr_dinfo2(syscall, "vfs_readlinkat(dirfd=%d, path='%s', buf=%p, buf_size=%zu)", (fd_t) arg1, (const char *) arg2, (void *) arg3, (size_t) arg4);
            ret = (reg_t) impl_syscall_vfs_readlinkat((fd_t) arg1, (const char *) arg2, (char *) arg3, (size_t) arg4);
            break;
        }
        case SYSCALL_vfs_unlinkat:
        {
            pr_dinfo2(syscall, "vfs_unlinkat(dirfd=%d, path='%s')", (fd_t) arg1, (const char *) arg2);
            ret = (reg_t) impl_syscall_vfs_unlinkat((fd_t) arg1, (const char *) arg2);
            break;
        }
        case SYSCALL_vfs_symlink:
        {
            pr_dinfo2(syscall, "vfs_symlink(link_path='%s', target='%s')", (const char *) arg1, (const char *) arg2);
            ret = (reg_t) impl_syscall_vfs_symlink((const char *) arg1, (const char *) arg2);
            break;
        }
        case SYSCALL_vfs_mkdir:
        {
            pr_dinfo2(syscall, "vfs_mkdir(path='%s')", (const char *) arg1);
            ret = (reg_t) impl_syscall_vfs_mkdir((const char *) arg1);
            break;
        }
        case SYSCALL_vfs_list_dir:
        {
            pr_dinfo2(syscall, "vfs_list_dir(fd=%d, buffer=%p, buffer_size=%zu)", (fd_t) arg1, (void *) arg2, (size_t) arg3);
            ret = (reg_t) impl_syscall_vfs_list_dir((fd_t) arg1, (char *) arg2, (size_t) arg3);
            break;
        }
        case SYSCALL_fd_manipulate:
        {
            pr_dinfo2(syscall, "fd_manipulate(fd=%d, cmd=%llu, arg=%p)", (fd_t) arg1, (u64) arg2, (void *) arg3);
            ret = (reg_t) impl_syscall_fd_manipulate((fd_t) arg1, (u64) arg2, (void *) arg3);
            break;
        }
        case SYSCALL_mmap_anonymous:
        {
            pr_dinfo2(syscall, "mmap_anonymous(hint_addr=" PTR_FMT ", size=%zu, perm=%d, flags=%d)", (ptr_t) arg1, (size_t) arg2, (mem_perm_t) arg3, (mmap_flags_t) arg4);
            ret = (reg_t) impl_syscall_mmap_anonymous((ptr_t) arg1, (size_t) arg2, (mem_perm_t) arg3, (mmap_flags_t) arg4);
            break;
        }
        case SYSCALL_mmap_file:
        {
            pr_dinfo2(syscall, "mmap_file(hint_addr=" PTR_FMT ", size=%zu, perm=%d, flags=%d, fd=%d, offset=%zd)", (ptr_t) arg1, (size_t) arg2, (mem_perm_t) arg3, (mmap_flags_t) arg4, (fd_t) arg5, (off_t) arg6);
            ret = (reg_t) impl_syscall_mmap_file((ptr_t) arg1, (size_t) arg2, (mem_perm_t) arg3, (mmap_flags_t) arg4, (fd_t) arg5, (off_t) arg6);
            break;
        }
        case SYSCALL_wait_for_process:
        {
            pr_dinfo2(syscall, "wait_for_process(pid=%d, exit_code=%p, flags=%u)", (pid_t) arg1, (void *) arg2, (u32) arg3);
            ret = (reg_t) impl_syscall_wait_for_process((pid_t) arg1, (u32 *) arg2, (u32) arg3);
            break;
        }
        case SYSCALL_munmap:
        {
            pr_dinfo2(syscall, "munmap(addr=%p, size=%zu)", (void *) arg1, (size_t) arg2);
            ret = (reg_t) impl_syscall_munmap((void *) arg1, (size_t) arg2);
            break;
        }
        case SYSCALL_vfs_chdirat:
        {
            pr_dinfo2(syscall, "vfs_chdirat(dirfd=%d, path='%s')", (fd_t) arg1, (const char *) arg2);
            ret = (reg_t) impl_syscall_vfs_chdirat((fd_t) arg1, (const char *) arg2);
            break;
        }
        case SYSCALL_vfs_getcwd:
        {
            pr_dinfo2(syscall, "vfs_getcwd(buf=%p, buf_size=%zu)", (void *) arg1, (size_t) arg2);
            ret = (reg_t) impl_syscall_vfs_getcwd((char *) arg1, (size_t) arg2);
            break;
        }
        case SYSCALL_io_seek:
        {
            pr_dinfo2(syscall, "io_seek(fd=%d, offset=%zd, whence=%d)", (fd_t) arg1, (off_t) arg2, (io_seek_whence_t) arg3);
            ret = (reg_t) impl_syscall_io_seek((fd_t) arg1, (off_t) arg2, (io_seek_whence_t) arg3);
            break;
        }
        case SYSCALL_io_tell:
        {
            pr_dinfo2(syscall, "io_tell(fd=%d)", (fd_t) arg1);
            ret = (reg_t) impl_syscall_io_tell((fd_t) arg1);
            break;
        }
        case SYSCALL_signal_register:
        {
            pr_dinfo2(syscall, "signal_register(signum=%d, action=%p)", (signal_t) arg1, (void *) arg2);
            ret = (reg_t) impl_syscall_signal_register((signal_t) arg1, (const sigaction_t *) arg2);
            break;
        }
        case SYSCALL_signal_process:
        {
            pr_dinfo2(syscall, "signal_process(pid=%d, signum=%d)", (pid_t) arg1, (signal_t) arg2);
            ret = (reg_t) impl_syscall_signal_process((pid_t) arg1, (signal_t) arg2);
            break;
        }
        case SYSCALL_signal_thread:
        {
            pr_dinfo2(syscall, "signal_thread(tid=%d, signum=%d)", (tid_t) arg1, (signal_t) arg2);
            ret = (reg_t) impl_syscall_signal_thread((tid_t) arg1, (signal_t) arg2);
            break;
        }
        case SYSCALL_poweroff:
        {
            pr_dinfo2(syscall, "poweroff(reboot=%d, magic=%u)", (bool) arg1, (u32) arg2);
            impl_syscall_poweroff((bool) arg1, (u32) arg2);
            break;
        }
        case SYSCALL_signal_return:
        {
            pr_dinfo2(syscall, "signal_return(sp=%p)", (void *) arg1);
            impl_syscall_signal_return((void *) arg1);
            break;
        }
        case SYSCALL_vm_protect:
        {
            pr_dinfo2(syscall, "vm_protect(addr=%p, size=%zu, perm=%d)", (void *) arg1, (size_t) arg2, (mem_perm_t) arg3);
            ret = (reg_t) impl_syscall_vm_protect((void *) arg1, (size_t) arg2, (mem_perm_t) arg3);
            break;
        }
        case SYSCALL_io_poll:
        {
            pr_dinfo2(syscall, "io_poll(fds=%p, nfds=%ld, timeout=%d)", (void *) arg1, (nfds_t) arg2, (int) arg3);
            ret = (reg_t) impl_syscall_io_poll((struct pollfd *) arg1, (nfds_t) arg2, (int) arg3);
            break;
        }
        case SYSCALL_io_pselect:
        {
            pr_dinfo2(syscall, "io_pselect(nfds=%d, readfds=%p, writefds=%p, exceptfds=%p, timeout=%p, sigmask=%p)", (int) arg1, (void *) arg2, (void *) arg3, (void *) arg4, (void *) arg5, (void *) arg6);
            ret = (reg_t) impl_syscall_io_pselect((int) arg1, (fd_set *) arg2, (fd_set *) arg3, (fd_set *) arg4, (const struct timespec *) arg5, (const sigset_t *) arg6);
            break;
        }
        case SYSCALL_execveat:
        {
            pr_dinfo2(syscall, "execveat(dirfd=%d, file_path='%s', argv=%p, envp=%p, flags=%u)", (fd_t) arg1, (const char *) arg2, (void *) arg3, (void *) arg4, (u32) arg5);
            ret = (reg_t) impl_syscall_execveat((fd_t) arg1, (const char *) arg2, (const char *const *) arg3, (const char *const *) arg4, (u32) arg5);
            break;
        }
        case SYSCALL_clock_msleep:
        {
            pr_dinfo2(syscall, "clock_msleep(ms=%llu)", (u64) arg1);
            impl_syscall_clock_msleep((u64) arg1);
            break;
        }
        case SYSCALL_io_dup:
        {
            pr_dinfo2(syscall, "io_dup(fd=%d)", (fd_t) arg1);
            ret = (reg_t) impl_syscall_io_dup((fd_t) arg1);
            break;
        }
        case SYSCALL_io_dup2:
        {
            pr_dinfo2(syscall, "io_dup2(old_fd=%d, new_fd=%d)", (fd_t) arg1, (fd_t) arg2);
            ret = (reg_t) impl_syscall_io_dup2((fd_t) arg1, (fd_t) arg2);
            break;
        }
        case SYSCALL_dmabuf_alloc:
        {
            pr_dinfo2(syscall, "dmabuf_alloc(n_pages=%zu, out_paddr=%p, out_vaddr=%p)", (size_t) arg1, (void *) arg2, (void *) arg3);
            ret = (reg_t) impl_syscall_dmabuf_alloc((size_t) arg1, (ptr_t *) arg2, (ptr_t *) arg3);
            break;
        }
        case SYSCALL_dmabuf_free:
        {
            pr_dinfo2(syscall, "dmabuf_free(vaddr=" PTR_FMT ", paddr=" PTR_FMT ")", (ptr_t) arg1, (ptr_t) arg2);
            ret = (reg_t) impl_syscall_dmabuf_free((ptr_t) arg1, (ptr_t) arg2);
            break;
        }
        case SYSCALL_dmabuf_share:
        {
            pr_dinfo2(syscall, "dmabuf_share(buf=%p, bufsize=%zu, out_paddr=%p)", (void *) arg1, (size_t) arg2, (void *) arg3);
            ret = (reg_t) impl_syscall_dmabuf_share((void *) arg1, (size_t) arg2, (ptr_t *) arg3);
            break;
        }
        case SYSCALL_dmabuf_unshare:
        {
            pr_dinfo2(syscall, "dmabuf_unshare(paddr=" PTR_FMT ", size=%zu, vaddr=%p)", (ptr_t) arg1, (size_t) arg2, (void *) arg3);
            ret = (reg_t) impl_syscall_dmabuf_unshare((ptr_t) arg1, (size_t) arg2, (void *) arg3);
            break;
        }
        case SYSCALL_pipe:
        {
            pr_dinfo2(syscall, "pipe(out_read_fd=%p, out_write_fd=%p, flags=%d)", (void *) arg1, (void *) arg2, (fd_flags_t) arg3);
            ret = (reg_t) impl_syscall_pipe((fd_t *) arg1, (fd_t *) arg2, (fd_flags_t) arg3);
            break;
        }
        case SYSCALL_io_readv:
        {
            pr_dinfo2(syscall, "io_readv(fd=%d, iov=%p, iov_count=%d)", (fd_t) arg1, (void *) arg2, (int) arg3);
            ret = (reg_t) impl_syscall_io_readv((fd_t) arg1, (const struct iovec *) arg2, (int) arg3);
            break;
        }
        case SYSCALL_vfs_unmount:
        {
            pr_dinfo2(syscall, "vfs_unmount(mount_point='%s')", (const char *) arg1);
            ret = (reg_t) impl_syscall_vfs_unmount((const char *) arg1);
            break;
        }
        case SYSCALL_clock_gettimeofday:
        {
            pr_dinfo2(syscall, "clock_gettimeofday(tv=%p)", (void *) arg1);
            ret = (reg_t) impl_syscall_clock_gettimeofday((struct timespec *) arg1);
            break;
        }
        case SYSCALL_thread_setname:
        {
            pr_dinfo2(syscall, "thread_setname(tid=%d, name='%s')", (tid_t) arg1, (const char *) arg2);
            ret = (reg_t) impl_syscall_thread_setname((tid_t) arg1, (const char *) arg2);
            break;
        }
        case SYSCALL_thread_getname:
        {
            pr_dinfo2(syscall, "thread_getname(tid=%d, buf=%p, buf_size=%zu)", (tid_t) arg1, (void *) arg2, (size_t) arg3);
            ret = (reg_t) impl_syscall_thread_getname((tid_t) arg1, (char *) arg2, (size_t) arg3);
            break;
        }
        case SYSCALL_vfs_fchmodat:
        {
            pr_dinfo2(syscall, "vfs_fchmodat(dirfd=%d, path='%s', mode=%d, flags=%d)", (fd_t) arg1, (const char *) arg2, (int) arg3, (int) arg4);
            ret = (reg_t) impl_syscall_vfs_fchmodat((fd_t) arg1, (const char *) arg2, (int) arg3, (int) arg4);
            break;
        }
        case SYSCALL_io_pread:
        {
            pr_dinfo2(syscall, "io_pread(fd=%d, buf=%p, count=%zu, offset=%zd)", (fd_t) arg1, (void *) arg2, (size_t) arg3, (off_t) arg4);
            ret = (reg_t) impl_syscall_io_pread((fd_t) arg1, (void *) arg2, (size_t) arg3, (off_t) arg4);
            break;
        }
        case SYSCALL_memfd_create:
        {
            pr_dinfo2(syscall, "memfd_create(name='%s', flags=%u)", (const char *) arg1, (u32) arg2);
            ret = (reg_t) impl_syscall_memfd_create((const char *) arg1, (u32) arg2);
            break;
        }
        case SYSCALL_signal_mask_op:
        {
            pr_dinfo2(syscall, "signal_mask_op(how=%d, set=%p, oldset=%p)", (int) arg1, (void *) arg2, (void *) arg3);
            ret = (reg_t) impl_syscall_signal_mask_op((int) arg1, (const sigset_t *) arg2, (sigset_t *) arg3);
            break;
        }
    }
    
    pr_dcont(syscall, " = 0x%lx", ret);
    return ret;
}
